@@ -6,7 +6,7 @@ pub mod serve;
 use bonsaidb_local::cli::StorageCommand;
 use clap::Parser;
 
-use crate::{Backend, CustomServer, Error, NoBackend, ServerConfiguration};
+use crate::{Backend, BackendError, CustomServer, NoBackend, ServerConfiguration};
 
 /// Available commands for `bonsaidb server`.
 #[derive(Parser, Debug)]
@@ -25,12 +25,18 @@ pub enum Command<B: Backend = NoBackend> {
 
 impl<B: Backend> Command<B> {
     /// Executes the command.
-    pub async fn execute(&self, configuration: ServerConfiguration) -> Result<(), Error> {
+    pub async fn execute(
+        &self,
+        configuration: ServerConfiguration<B>,
+    ) -> Result<(), BackendError<B::Error>> {
         let server = CustomServer::<B>::open(configuration).await?;
         match self {
             Self::Certificate(command) => command.execute(&server).await,
             Self::Serve(command) => command.execute(&server).await,
-            Self::Storage(command) => command.execute_on(&server).await.map_err(Error::from),
+            Self::Storage(command) => command
+                .execute_on_async(&server)
+                .await
+                .map_err(BackendError::from),
         }
     }
 }
