@@ -1,18 +1,17 @@
 use bonsaidb_core::{
     define_basic_mapped_view, define_basic_unique_mapped_view,
-    document::{CollectionDocument, KeyId},
+    document::{CollectionDocument, Emit, KeyId},
     schema::{Collection, NamedCollection, Schema},
 };
 use fabruic::{CertificateChain, PrivateKey};
 use serde::{de::Visitor, Deserialize, Serialize};
 
 #[derive(Debug, Schema)]
-#[schema(name = "hosted", authority = "khonsulabs", core = bonsaidb_core)]
-#[cfg_attr(feature = "acme", schema(collections = [TlsCertificate, crate::server::acme::AcmeAccount]))]
-#[cfg_attr(not(feature = "acme"), schema(collections = [TlsCertificate]))]
+#[schema(name = "hosted", authority = "khonsulabs", collections = [TlsCertificate], core = bonsaidb_core)]
+#[cfg_attr(feature = "acme", schema(collections = [crate::server::acme::AcmeAccount]))]
 pub struct Hosted;
 
-#[derive(Debug, Serialize, Deserialize, Collection)]
+#[derive(Clone, Debug, Serialize, Deserialize, Collection)]
 #[collection(name = "tls-certificates", authority = "khonsulabs", views = [TlsCertificatesByDomain, TlsCertificateByAllDomains])]
 #[collection(encryption_key = Some(KeyId::Master), encryption_optional, core = bonsaidb_core)]
 pub struct TlsCertificate {
@@ -32,7 +31,9 @@ define_basic_unique_mapped_view!(
     "by-all-domains",
     String,
     |document: CollectionDocument<TlsCertificate>| {
-        document.emit_key(document.contents.domains.join(";"))
+        document
+            .header
+            .emit_key(document.contents.domains.join(";"))
     }
 );
 
@@ -57,7 +58,7 @@ define_basic_mapped_view!(
     }
 );
 
-#[derive(Debug)]
+#[derive(Clone, Debug)]
 pub struct SerializablePrivateKey(pub PrivateKey);
 
 impl Serialize for SerializablePrivateKey {
